"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.isOptions = void 0;
const stream_1 = require("stream");
const dropbox_1 = require("dropbox");
const node_fetch_1 = __importDefault(require("node-fetch"));
const Util = __importStar(require("./util"));
function isOptions(o) {
    return typeof o === 'object' && typeof o.token === 'string';
}
exports.isOptions = isOptions;
/**
 * Input paths can look like this:
 *      Root application directory: '/'
 *      Subdirectory: 'subdirectory'
 *
 * The corresponding paths to the Dropbox API need to look like this:
 *      Root application directory: ''
 *      Subdirectory: '/subdirectory'
 */
function coercePath(path) {
    if (path === '/' || path === '')
        return '';
    if (!path.startsWith('/'))
        return `/${path}`;
    return path;
}
function isDirectory(m) {
    return m['.tag'] === 'folder';
}
function isFile(m) {
    return m['.tag'] === 'file';
}
function create({ token: accessToken, }) {
    const dbx = new dropbox_1.Dropbox({ accessToken, fetch: node_fetch_1.default });
    async function read(path) {
        const response = await dbx.filesDownload({
            path: coercePath(path),
        });
        const result = response.result;
        if (!result.fileBinary) {
            throw new Error('Missing file contents in Dropbox response');
        }
        return result.fileBinary;
    }
    async function write(path, contents) {
        await dbx.filesUpload({ path: coercePath(path), contents });
    }
    // This API doesn't actually support streaming, so we'll just pass through to
    // the normal read/write for compatibility.
    async function createReadStream(path) {
        const buffer = await read(path);
        return stream_1.Readable.from(buffer);
    }
    async function writeStream(path, stream) {
        const data = await Util.drain(stream);
        return write(path, data);
    }
    async function list(path) {
        try {
            const response = await dbx.filesListFolder({ path: coercePath(path) });
            const dirs = response.result.entries
                .filter(isDirectory)
                .map(({ name }) => ({ path: name, type: 'directory' }));
            const files = response.result.entries
                .filter(isFile)
                .map(({ name, size }) => ({ path: name, type: 'file', size }));
            return [...dirs, ...files].sort((a, b) => a.path.localeCompare(b.path));
        }
        catch (e) {
            throw new Error('Failed to list dropbox contents');
        }
    }
    return { list, read, write, createReadStream, writeStream };
}
exports.create = create;
