"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.remove = exports.list = exports.writeStream = exports.createReadStream = exports.write = exports.read = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
async function read(path) {
    return fs_1.default.promises.readFile(path);
}
exports.read = read;
async function write(path, data) {
    return fs_1.default.promises.writeFile(path, data);
}
exports.write = write;
async function createReadStream(path) {
    await fs_1.default.promises.access(path);
    return fs_1.default.createReadStream(path);
}
exports.createReadStream = createReadStream;
async function writeStream(path, data) {
    await new Promise((resolve, reject) => {
        const w = fs_1.default.createWriteStream(path);
        w.on('close', resolve);
        w.on('error', reject);
        data.pipe(w);
    });
}
exports.writeStream = writeStream;
async function list(path) {
    // We'll keep the behavior here simple for now and skip oddities like FIFOs,
    // sockets, block devices, symlinks, etc.  We could follow symlinks but for
    // now we'll skip those too.
    const list = (await fs_1.default.promises.readdir(path, { withFileTypes: true }))
        .filter((dirent) => dirent.isFile() || dirent.isDirectory())
        .map((dirent) => ({
        path: dirent.name,
        type: dirent.isDirectory() ? 'directory' : 'file',
    }));
    return Promise.all(list.map(async (entry) => {
        if (entry.type === 'directory')
            return entry;
        const { size } = await fs_1.default.promises.stat(path_1.join(path, entry.path));
        return { ...entry, size };
    }));
}
exports.list = list;
async function remove(path) {
    await fs_1.default.promises.unlink(path);
}
exports.remove = remove;
function create() {
    return {
        read,
        write,
        createReadStream,
        writeStream,
        list,
        remove,
    };
}
exports.create = create;
