"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = exports.readText = exports.readJson = exports.createReadStream = exports.read = exports.ResponseError = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
class ResponseError extends Error {
    constructor(code, text, body) {
        super(text);
        this.code = code;
        this.body = body;
    }
}
exports.ResponseError = ResponseError;
async function run(url, options) {
    const response = await node_fetch_1.default(url, options);
    const { ok, status, statusText } = response;
    if (!ok) {
        const type = response.headers.get('content-type');
        const body = type && type.startsWith('application/json')
            ? await response.json()
            : await response.text();
        throw new ResponseError(status, statusText, body);
    }
    return response;
}
async function read(path) {
    return (await run(path)).buffer();
}
exports.read = read;
async function createReadStream(path) {
    return (await run(path)).body;
}
exports.createReadStream = createReadStream;
async function readJson(path) {
    return (await run(path)).json();
}
exports.readJson = readJson;
async function readText(path) {
    return (await run(path)).text();
}
exports.readText = readText;
function create(protocol) {
    function prefix(path) {
        return `${protocol}://${path}`;
    }
    async function protoRead(path) {
        return read(prefix(path));
    }
    async function protoCreateReadStream(path) {
        return createReadStream(prefix(path));
    }
    return {
        read: protoRead,
        createReadStream: protoCreateReadStream,
    };
}
exports.create = create;
